#include <Quickdraw.h>
#include <ControlMgr.h>

/* program to demonstrate how to write a control definition
 * function.
 * Steve Hawley  8/88
 * LightSpeed C v. 2.01
 * Portions are copyright Think Technologies...
 */

/* all control defs start with main in LSC */
pascal long
main(varCode, theControl, message, param)
int varCode;
ControlHandle theControl;
int message;
long param;
{
	long myTestCtrl(), myDragCtrl();
	
	switch (message) {
	case drawCntl:
		if ((**theControl).contrlVis)
			myDrawCtrl(varCode, theControl, param);
		break;
	case testCntl:
		return(myTestCtrl(theControl, param));
		break;
	case calcCRgns: /* no region to define */
		break;
	case initCntl:	/* no initialization */
		break;
	case dispCntl:	/* no disposal */
		break;
	case posCntl:	/* position can look just like draw */
		if ((**theControl).contrlVis)
			myDrawCtrl(varCode, theControl, param);
		break;
	case thumbCntl:	/* no thumb */
		break;
	case dragCntl:	/* tracks mouse down */
		return(myDragCtrl(theControl));
		break;
	case autoTrack:	/* no auto tracking */
		break;
	}
	return(0);
}

myDrawCtrl(varCode, theControl, param)
int varCode;
ControlHandle theControl;
long param;
{
	PenState pstate;
	PolyHandle lside, bside;
	unsigned int wqtr, hqtr, wsth, hsth;
	register ControlPtr cp;
	Rect bezel, top;
	FontInfo finfo;
	RgnHandle save_clip;
	
	HLock(theControl);	/* lock our data, just in case */
	cp = *theControl;
	
	GetPenState(&pstate);	/* save pen state to be nice */
	PenNormal();	/* make no assumptions about the state of things */
	if (cp->contrlHilite == 255) PenPat(gray);
	save_clip = NewRgn();
	GetClip(save_clip);
	
	GetFontInfo(&finfo);
	finfo.ascent += finfo.descent + finfo.leading;
	
	/* grid off Rect in Pixels */
	wqtr = (cp->contrlRect.right - cp->contrlRect.left)/4;
	wsth = wqtr/4;
	hqtr = (cp->contrlRect.bottom - cp->contrlRect.top)/4;
	hsth = wqtr/4;
	
	SetRect(&bezel, cp->contrlRect.left, cp->contrlRect.top + hsth,
			cp->contrlRect.right - wsth, cp->contrlRect.bottom);
	/* button out */		
	if (cp->contrlValue == 0) {
		SetRect(&top, cp->contrlRect.left + wqtr, cp->contrlRect.top,
			cp->contrlRect.right, cp->contrlRect.bottom - hqtr);
	
		lside = OpenPoly();
		MoveTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.top + hqtr/2);
		LineTo(cp->contrlRect.left + wqtr, cp->contrlRect.top);
		LineTo(cp->contrlRect.left + wqtr, cp->contrlRect.bottom - hqtr - 1);
		LineTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.bottom - hqtr/2 - 1);
		LineTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.top + hqtr/2);
		ClosePoly();
	
		bside = OpenPoly();
		MoveTo(cp->contrlRect.left + wqtr, cp->contrlRect.bottom - hqtr-1);
		LineTo(cp->contrlRect.right-1, cp->contrlRect.bottom - hqtr-1);
		LineTo(cp->contrlRect.right - wqtr/2 - 1, cp->contrlRect.bottom - hqtr/2 - 1);
		LineTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.bottom - hqtr/2 - 1);
		LineTo(cp->contrlRect.left + wqtr, cp->contrlRect.bottom - hqtr-1);
		ClosePoly();
	}
	else { /* button in */
		SetRect(&top, cp->contrlRect.left + wqtr-wsth, cp->contrlRect.top+hsth,
			cp->contrlRect.right-wsth, cp->contrlRect.bottom - hqtr+hsth);
	
		lside = OpenPoly();
		MoveTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.top + hqtr/2);
		LineTo(cp->contrlRect.left + wqtr-wsth, cp->contrlRect.top+hsth);
		LineTo(cp->contrlRect.left + wqtr-wsth, cp->contrlRect.bottom - hqtr+hsth - 1);
		LineTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.bottom - hqtr/2 - 1);
		LineTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.top + hqtr/2);
		ClosePoly();
	
		bside = OpenPoly();
		MoveTo(cp->contrlRect.left + wqtr-wsth, cp->contrlRect.bottom - hqtr+hsth - 1);
		LineTo(cp->contrlRect.right-wsth-1, cp->contrlRect.bottom - hqtr+hsth - 1);
		LineTo(cp->contrlRect.right - wqtr/2 - 1, cp->contrlRect.bottom - hqtr/2 - 1);
		LineTo(cp->contrlRect.left + wqtr/2, cp->contrlRect.bottom - hqtr/2 - 1);
		LineTo(cp->contrlRect.left + wqtr, cp->contrlRect.bottom - wqtr - 1);
		ClosePoly();
	}
	
	EraseRect(&cp->contrlRect);	/* clear space */
	FrameRect(&bezel);
	ErasePoly(lside);
	FramePoly(lside);
	ErasePoly(bside);
	FramePoly(bside);
	EraseRect(&top);
	FrameRect(&top);
	ClipRect(&top);
	/* draw text clipped to button top */
	MoveTo((top.left + top.right - StringWidth(cp->contrlTitle))/2,
		top.top + (top.bottom - top.top + finfo.ascent)/2);
	DrawString(cp->contrlTitle);
	SetClip(save_clip);
	if (cp->contrlValue == 1) { /* hilite button top if pressed in */
		InsetRect(&top, 1, 1);
		InvertRect(&top);
	}
	
	DisposeRgn(save_clip); /* clean up */
	KillPoly(lside);
	KillPoly(bside);
	SetPenState(&pstate);
	HUnlock(theControl);
}

long
myTestCtrl(theControl, pt)
ControlHandle theControl;
Point pt;
{
	register ControlPtr cp;
	Rect top;
	unsigned int wqtr, hqtr, wsth, hsth;
	
	cp = *theControl;
	
	wqtr = (cp->contrlRect.right - cp->contrlRect.left)/4;
	wsth = wqtr/4;
	hqtr = (cp->contrlRect.bottom - cp->contrlRect.top)/4;
	hsth = wqtr/4;
	
	if (cp->contrlValue == 0) {
		SetRect(&top, cp->contrlRect.left + wqtr, cp->contrlRect.top,
			cp->contrlRect.right, cp->contrlRect.bottom - hqtr);
	}
	else {
		SetRect(&top, cp->contrlRect.left + wqtr-wsth, cp->contrlRect.top+hsth,
			cp->contrlRect.right-wsth, cp->contrlRect.bottom - hqtr+hsth);
	}
	/* if in top part of button, return 129 (standard part code ) */
	if(PtInRect(pt, &top)) return((long)129);
	return((long)0);
}

long
myDragCtrl(theControl)
ControlHandle theControl;
{
	register ControlPtr cp;
	Rect top;
	unsigned int wqtr, hqtr, wsth, hsth;
	register unsigned int invert;
	Point apoint;
	
	cp = *theControl;
	
	wqtr = (cp->contrlRect.right - cp->contrlRect.left)/4;
	wsth = wqtr/4;
	hqtr = (cp->contrlRect.bottom - cp->contrlRect.top)/4;
	hsth = wqtr/4;
	
	if (cp->contrlValue == 0) {
		SetRect(&top, cp->contrlRect.left + wqtr, cp->contrlRect.top,
			cp->contrlRect.right, cp->contrlRect.bottom - hqtr);
	}
	else {
		SetRect(&top, cp->contrlRect.left + wqtr-wsth, cp->contrlRect.top+hsth,
			cp->contrlRect.right-wsth, cp->contrlRect.bottom - hqtr+hsth);
	}
	
	InsetRect(&top, 1, 1); /* start lit */
	invert = 1;
	InvertRect(&top);
	
	while(Button()) { /* track mouse */
		GetMouse(&apoint);
		if (invert && !PtInRect(apoint, &top)) {
			invert = 0;
			InvertRect(&top);
		}
		else if (!invert && PtInRect(apoint, &top)) {
			invert = 1;
			InvertRect(&top);
		}
	}
	
	if (invert) { /* selected, set the appropriate control value */
		InvertRect(&top);
		if (cp->contrlValue == 0) SetCtlValue(theControl, 1);
		else SetCtlValue(theControl, 0);
	}
}